;PROBE gets info about nested entities at all levels of nesting, to allow you
; to list info about an entity in an insert without exploding the insert.
(defun C:PROBE( / datlst blnams blname basent ntnsel datlst )
 (setq datlst nil basent T)
 (princ "\nSelect entity to list: ")
 (if (setq ntnsel (nentsel))
  (progn
   (addata (car ntnsel))
   (if (setq blnams (cadddr ntnsel))
    (foreach blname blnams
     (addata blname)
    )
   )
   (if (< (setq dcl_id (load_dialog "probe.dcl")) 0)
    (princ "\n\007 PROBE.DCL not found! ")
    (if (null (new_dialog "probe_dlg" dcl_id))
     (princ "\nError in NEW_DIALOG. ")
     (progn
      (action_tile "cancel" "(done_dialog 0)")
      (action_tile "help" "(do_probe_help)")
      (action_tile "info" "(do_about_probe)")
      (start_list "data_box")
      (mapcar 'add_list (reverse datlst))
      (end_list)
      (start_dialog)
     )
    )
   )
  )
  (princ "\nNo entity selected. ")
 )
 (prin1)
)  

(defun addata(ename / etype edata datstr strlst ecolor )
 (if (setq edata (entget ename))
  (progn
   (setq etype (cdr (assoc 0 edata)))
   (setq datstr
    (strcat
     (if basent
      (progn
       (setq basent nil)
       "   BASE ENTITY: "
      )
      "   PART OF: "
     )
     (cond
      ((= "INSERT" etype)
       (if
        (=
         0 
         (logand
          4 
          (cdr (assoc 70 (tblsearch "BLOCK" (cdr (assoc 2 edata)))))
         )
        )
        (strcat "INSERT named " (cdr (assoc 2 edata)))
        (strcat "XREF named " (cdr (assoc 2 edata)))
       )
      )
      ((= "TEXT" etype)
       (strcat
        "TEXT in style " 
        (if (cdr (assoc 7 edata))
         (cdr (assoc 7 edata))
         "STANDARD"
        )
       )
      )
      ((= "ATTRIB" etype)
       (strcat
        "ATTRIB in style " 
        (if (cdr (assoc 7 edata))
         (cdr (assoc 7 edata))
         "STANDARD"
        )
        ", with tag "
        (cdr (assoc 2 edata))
       )
      )
      (T
       etype
      )
     )
     ", on layer "
     (cdr (assoc 8 edata))
     ", in "
     (if (cdr (assoc 6 edata))
      (cdr (assoc 6 edata))
      "BYLAYER"
     )
     " linetype, in "
     (if (setq ecolor (cdr (assoc 62 edata)))
      (cond
       ((= ecolor 256) "BYBLOCK")
       ((= ecolor 0) "BYLAYER")
       ((= ecolor 1) "RED")
       ((= ecolor 2) "YELLOW")
       ((= ecolor 3) "GREEN")
       ((= ecolor 4) "CYAN")
       ((= ecolor 5) "BLUE")
       ((= ecolor 6) "MAGENTA")
       ((= ecolor 7) "WHITE")
       (T (itoa ecolor))
      )
      "BYLAYER"
     )
     " color. "
    )
   )
   (setq strlst (wrdwrp datstr 47))
   (if strlst
    (foreach substr (reverse strlst)
     (setq datlst (cons substr datlst))
    )
   )
  )
 )
)

(defun helpomat(hlplst hlpttl)
 (if (null (new_dialog "help_box" dcl_id))
  (progn
   (princ "\nError in NEW_DIALOG. ")
   (exit)
  )
 )
 (start_list "list_box")
 (mapcar 'add_list hlplst)
 (end_list)
 (set_tile "title" hlpttl)
 (start_dialog)
)

(defun do_about_probe()
 (helpomat
  (list
   " PROBE version 1.001"
   ""
   " PROBE.LSP gets info about nested entities at all"
   " levels of nesting."
   ""
   " Yet another piece of"
   ""
   "        T E R R I B L E   S O F T W A R E"
   ""
   "              Copyright (c) 1993 Terrible Software, Inc."
   ""
   "                              (that's ME, W. D. Kiernan)"
  )
  " About PROBE "
 )
)

(defun do_probe_help()
 (helpomat
  (list
   "---------------------------------------------------------"
   "PROBE help"
   "---------------------------------------------------------"
   "     PROBE is a program that tells you information about"
   " nested entities; that is, entities that are part of"
   " other entities, as a LINE entity might be a component"
   " of an INSERT, which in turn might be a component of"
   " an XREF.  If you want, for example, to turn off that"
   " LINE's visibilty, you might not know what layer to turn"
   " off.  If you LIST the LINE by picking it, LIST will tell"
   " you the name of the XREF and the layer it is on, but"
   " you still won't know what layer the LINE itself is on."
   " PROBE tells you all the various nested items and their"
   " layers, without forcing you to exit the drawing you're"
   " in and opening the XREF drawing, or exploding the block."
   "      Debbie Larson came up with the idea for this"
   " program.  I may add more information to the listing"
   " as the need arises.  Any of your suggestions for more"
   " features or information will be welcome. "
   "---------------------------------------------------------"
   "->   Advertisement"
   "     PROBE is Freeware.  Distribute freely with gusto! (I"
   " stole that line from Gen Kiyooka.) If you REALLY like it"
   " a whole lot send big bucks and/or job offers to"
   "                     William Kiernan"
   "                  10345 Del Mar Circle"
   "                     Tampa Fl 33624"
   " Any and all suggestions, criticism, neat hacks, etc. are"
   " welcome.  My CompuServe ID is 71331,3510"
   "     Incidentally, TERRIBLE SOFTWARE, INC. (that's me),"
   " as you could probably tell by the company name, assumes"
   " absolutely NO liability for losses, damages, floods,"
   " fires, tornadoes, plagues, etc., arising from the use"
   " of this program."
   "            (c) Terrible Software  /  WDK  /  4 Feb 1993"
  )
  " PROBE Help "
 )
)

(defun wrdwrp(string length / stpart stlist scount )
 (setq stlist nil)
 (while (and (/= "" string)(= " " (substr string (strlen string))))
  (setq string (substr string 1 (1- (strlen string))))
 )
 (if (or (= string "")(<= length 0))
  nil
  (progn
   (while (> (strlen string) length)
    (setq scount (1+ length) stpart nil)
    (while (and (null stpart)(> (setq scount (1- scount)) 1))
     (if (= " " (substr string scount 1))
      (setq stpart (substr string 1 (1- scount))
            string (substr string (1+ scount))
      )
     )
    )
    (if stpart
     (setq stlist (cons stpart stlist))
     (setq stlist (cons (substr string 1 length) stlist)
           string (substr string (1+ length))
     )
    )
    (while (and (/= "" string)(= " " (substr string 1 1)))
     (setq string (substr string 2))
    )
   )
   (if (/= "" string)
    (setq stlist (cons string stlist))
   )
  )
 )
)    
